package dk.nsp.idsas.integration.sores;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import dk.nsp.idsas.integration.util.ServiceErrorCallback;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.StringEntity;
import org.apache.log4j.Logger;

public class SoresClient {
    private static final String CONTENT_TYPE = "application/json";
    private static final Logger logger = Logger.getLogger(SoresClient.class);

    private final ObjectMapper objectMapper = new ObjectMapper();
    private final String url;
    private final HttpClient httpClient;
    private final ServiceErrorCallback serviceErrorCallback;

    public SoresClient(String url, HttpClient httpClient, ServiceErrorCallback serviceErrorCallback) {
        this.url = url;
        this.httpClient = httpClient;
        this.serviceErrorCallback = serviceErrorCallback;
    }

    public boolean cvrMapsToSor(String cpr) throws SoresClientException {
        try {
            HttpUriRequest request = buildPostRequest(cpr);
            return invokeService(request);
        } catch (JsonProcessingException | UnsupportedOperationException | UnsupportedEncodingException e) {
            throw new SoresClientException(e.getMessage(), e);
        }
    }

    private HttpUriRequest buildPostRequest(String inputCvr) throws JsonProcessingException, UnsupportedEncodingException {
        final HttpPost httpPost = new HttpPost(url);
        final Object requestBody = new Object() {
            public final String op = "EntityNameLookup";
            public final String matchEntityName = "";
            public final String cvr = inputCvr;
        };

        // using Jackson to serialize to JSON
        final ObjectMapper objectMapper = new ObjectMapper();
        final String jsonString = objectMapper.writeValueAsString(requestBody);
        final StringEntity entity = new StringEntity(jsonString);
        httpPost.setEntity(entity);
        httpPost.setHeader("Content-type", CONTENT_TYPE);
        return httpPost;
    }

    private boolean invokeService(HttpUriRequest request) throws SoresClientException {
        try {
            HttpResponse response = httpClient.execute(request);
            JsonNode jsonNode = objectMapper.readTree(response.getEntity().getContent());
            int responseSize = jsonNode.get("").size();
            logger.debug("SOR codes returned: " + responseSize);
            if (responseSize > 0) {
                serviceErrorCallback.registerServiceSuccess();
                return true;
            }
        } catch (IOException e) {
            serviceErrorCallback.registerServiceError();
            throw new SoresClientException(e.getMessage(), e);
        }
        return false;
    }

}
